Ext.data.JsonP.Ext_Version({"alternateClassNames":[],"aliases":{},"enum":null,"parentMixins":[],"tagname":"class","subclasses":[],"extends":null,"uses":[],"html":"<div><pre class=\"hierarchy\"><h4>Files</h4><div class='dependency'><a href='source/Version.html#Ext-Version' target='_blank'>Version.js</a></div></pre><div class='doc-contents'><p>A utility class that wraps around a version number string and provides convenient methods\nto perform comparisons. A version number is expressed in the following general format:</p>\n\n<pre><code>major[.minor[.patch[.build[release]]]]\n</code></pre>\n\n<p>The <code>Version</code> instance holds various readonly properties that contain the digested form\nof the version string. The numeric componnets of <code>major</code>, <code>minor</code>, <code>patch</code> and <code>build</code>\nas well as the textual suffix called <code>release</code>.</p>\n\n<p>Not depicted in the above syntax are three possible prefixes used to control partial\nmatching. These are '<sup>'</sup> (the default), '>' and '~'. These are discussed below.</p>\n\n<p>Examples:</p>\n\n<pre><code>var version = new <a href=\"#!/api/Ext.Version\" rel=\"Ext.Version\" class=\"docClass\">Ext.Version</a>('1.0.2beta'); // or maybe \"1.0\" or \"1.2.3.4RC\"\nconsole.log(\"Version is \" + version); // Version is 1.0.2beta\n\nconsole.log(version.getMajor()); // 1\nconsole.log(version.getMinor()); // 0\nconsole.log(version.getPatch()); // 2\nconsole.log(version.getBuild()); // 0\nconsole.log(version.getRelease()); // beta\n</code></pre>\n\n<p>The understood values of <code>release</code> are assigned numberic equivalents for the sake of\ncomparsion. The order of these from smallest to largest is as follows:</p>\n\n<ul>\n<li><code>\"dev\"</code></li>\n<li><code>\"alpha\"</code> or <code>\"a\"</code></li>\n<li><code>\"beta\"</code> or <code>\"b\"</code></li>\n<li><code>\"RC\"</code> or <code>\"rc\"</code></li>\n<li><code>\"#\"</code></li>\n<li><code>\"pl\"</code> or <code>\"p\"</code></li>\n</ul>\n\n\n<p>Any other (unrecognized) suffix is consider greater than any of these.</p>\n\n<h2>Comparisons</h2>\n\n<p>There are two forms of comparison that are commonly needed: full and partial. Full\ncomparison is simpler and is also the default.</p>\n\n<p>Example:</p>\n\n<pre><code>var version = new <a href=\"#!/api/Ext.Version\" rel=\"Ext.Version\" class=\"docClass\">Ext.Version</a>('1.0.2beta');\n\nconsole.log(version.isGreaterThan('1.0.1')); // True\nconsole.log(version.isGreaterThan('1.0.2alpha')); // True\nconsole.log(version.isGreaterThan('1.0.2RC')); // False\nconsole.log(version.isGreaterThan('1.0.2')); // False\nconsole.log(version.isLessThan('1.0.2')); // True\n\nconsole.log(version.match(1.0)); // True (using a Number)\nconsole.log(version.match('1.0.2')); // True (using a String)\n</code></pre>\n\n<p>These comparisons are ultimately implemented by <a href=\"#!/api/Ext.Version-method-compareTo\" rel=\"Ext.Version-method-compareTo\" class=\"docClass\">compareTo</a>\nwhich returns -1, 0 or 1 depending on whether the `Version' instance is less than, equal\nto, or greater than the given \"other\" version.</p>\n\n<p>For example:</p>\n\n<pre><code> var n = version.compareTo('1.0.1');  // == 1  (because 1.0.2beta &gt; 1.0.1)\n\n n = version.compareTo('1.1');  // == -1\n n = version.compareTo(version); // == 0\n</code></pre>\n\n<h3>Partial Comparisons</h3>\n\n<p>By default, unspecified version number fields are filled with 0. In other words, the\nversion number fields are 0-padded on the right or a \"lower bound\". This produces the\nmost commonly used forms of comparsion:</p>\n\n<pre><code> var ver = new Version('4.2');\n\n n = ver.compareTo('4.2.1'); // == -1  (4.2 promotes to 4.2.0 and is less than 4.2.1)\n</code></pre>\n\n<p>There are two other ways to interpret comparisons of versions of different length. The\nfirst of these is to change the padding on the right to be a large number (scuh as\nInfinity) instead of 0. This has the effect of making the version an upper bound. For\nexample:</p>\n\n<pre><code> var ver = new Version('^4.2'); // NOTE: the '^' prefix used\n\n n = ver.compreTo('4.3'); // == -1  (less than 4.3)\n\n n = ver.compareTo('4.2'); // == 1   (greater than all 4.2's)\n n = ver.compareTo('4.2.1'); // == 1\n n = ver.compareTo('4.2.9'); // == 1\n</code></pre>\n\n<p>The second way to interpret this comparison is to ignore the extra digits, making the\nmatch a prefix match. For example:</p>\n\n<pre><code> var ver = new Version('~4.2'); // NOTE: the '~' prefix used\n\n n = ver.compreTo('4.3'); // == -1\n\n n = ver.compareTo('4.2'); // == 0\n n = ver.compareTo('4.2.1'); // == 0\n</code></pre>\n\n<p>This final form can be useful when version numbers contain more components than are\nimportant for certain comparisons. For example, the full version of Ext JS 4.2.1 is\n\"4.2.1.883\" where 883 is the <code>build</code> number.</p>\n\n<p>This is how to create a \"partial\" <code>Version</code> and compare versions to it:</p>\n\n<pre><code> var version421ish = new Version('~4.2.1');\n\n n = version421ish.compareTo('4.2.1.883'); // == 0\n n = version421ish.compareTo('4.2.1.2'); // == 0\n n = version421ish.compareTo('4.2.1'); // == 0\n\n n = version421ish.compareTo('4.2'); // == 1\n</code></pre>\n\n<p>In the above example, '4.2.1.2' compares as equal to '4.2.1' because digits beyond the\ngiven \"4.2.1\" are ignored. However, '4.2' is less than the '4.2.1' prefix; its missing\ndigit is filled with 0.</p>\n</div><div class='members'><div class='members-section'><div class='definedBy'>Defined By</div><h3 class='members-title icon-property'>Properties</h3><div class='subsection'><div id='property-build' class='member first-child not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-property-build' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-property-build' class='name not-expandable'>build</a><span> : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></span><strong class='readonly signature' >readonly</strong></div><div class='description'><div class='short'><p>The fourth numeric part of the version number string.</p>\n</div><div class='long'><p>The fourth numeric part of the version number string.</p>\n</div></div></div><div id='property-major' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-property-major' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-property-major' class='name not-expandable'>major</a><span> : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></span><strong class='readonly signature' >readonly</strong></div><div class='description'><div class='short'><p>The first numeric part of the version number string.</p>\n</div><div class='long'><p>The first numeric part of the version number string.</p>\n</div></div></div><div id='property-minor' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-property-minor' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-property-minor' class='name not-expandable'>minor</a><span> : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></span><strong class='readonly signature' >readonly</strong></div><div class='description'><div class='short'><p>The second numeric part of the version number string.</p>\n</div><div class='long'><p>The second numeric part of the version number string.</p>\n</div></div></div><div id='property-parts' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-property-parts' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-property-parts' class='name expandable'>parts</a><span> : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a>[]</span><strong class='private signature' >private</strong><strong class='readonly signature' >readonly</strong></div><div class='description'><div class='short'>The split array of version number components found in the version string. ...</div><div class='long'><p>The split array of version number components found in the version string.\nFor example, for \"1.2.3\", this would be <code>[1, 2, 3]</code>.</p>\n</div></div></div><div id='property-patch' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-property-patch' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-property-patch' class='name not-expandable'>patch</a><span> : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></span><strong class='readonly signature' >readonly</strong></div><div class='description'><div class='short'><p>The third numeric part of the version number string.</p>\n</div><div class='long'><p>The third numeric part of the version number string.</p>\n</div></div></div><div id='property-release' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-property-release' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-property-release' class='name expandable'>release</a><span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a></span><strong class='readonly signature' >readonly</strong></div><div class='description'><div class='short'>The release level. ...</div><div class='long'><p>The release level. The following values are understood:</p>\n\n<ul>\n<li><code>\"dev\"</code></li>\n<li><code>\"alpha\"</code> or <code>\"a\"</code></li>\n<li><code>\"beta\"</code> or <code>\"b\"</code></li>\n<li><code>\"RC\"</code> or <code>\"rc\"</code></li>\n<li><code>\"#\"</code></li>\n<li><code>\"pl\"</code> or <code>\"p\"</code></li>\n</ul>\n\n<p>Defaults to: <code>&quot;&quot;</code></p></div></div></div></div></div><div class='members-section'><h3 class='members-title icon-method'>Methods</h3><div class='subsection'><div class='definedBy'>Defined By</div><h4 class='members-subtitle'>Instance Methods</h3><div id='method-constructor' class='member first-child not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-constructor' target='_blank' class='view-source'>view source</a></div><strong class='new-keyword'>new</strong><a href='#!/api/Ext.Version-method-constructor' class='name expandable'>Ext.Version</a>( <span class='pre'>version, [defaultMode]</span> ) : <a href=\"#!/api/Ext.Version\" rel=\"Ext.Version\" class=\"docClass\">Ext.Version</a></div><div class='description'><div class='short'> ...</div><div class='long'>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>version</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a><div class='sub-desc'><p>The version number.</p>\n</div></li><li><span class='pre'>defaultMode</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a> (optional)<div class='sub-desc'><p>The padding mode (e.g., '<sup>',</sup> '>' or '~').\nThis is ignored if the <code>version</code> contains an explicit mode prefix.</p>\n<p>Defaults to: <code>&quot;^&quot;</code></p></div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Ext.Version\" rel=\"Ext.Version\" class=\"docClass\">Ext.Version</a></span><div class='sub-desc'><p>this</p>\n</div></li></ul></div></div></div><div id='method-compareTo' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-compareTo' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-compareTo' class='name expandable'>compareTo</a>( <span class='pre'>other</span> ) : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></div><div class='description'><div class='short'>Compares this version instance to the specified other version. ...</div><div class='long'><p>Compares this version instance to the specified <code>other</code> version.</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>other</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a>/<a href=\"#!/api/Ext.Version\" rel=\"Ext.Version\" class=\"docClass\">Ext.Version</a><div class='sub-desc'><p>The other version to which to compare.</p>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></span><div class='sub-desc'><p>-1 if this version is less than the target version, 1 if this\nversion is greater, and 0 if they are equal.</p>\n</div></li></ul></div></div></div><div id='method-equals' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-equals' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-equals' class='name expandable'>equals</a>( <span class='pre'>target</span> ) : <a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></div><div class='description'><div class='short'>Returns whether this version equals to the supplied argument ...</div><div class='long'><p>Returns whether this version equals to the supplied argument</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>target</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a><div class='sub-desc'><p>The version to compare with</p>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></span><div class='sub-desc'><p>True if this version equals to the target, false otherwise</p>\n</div></li></ul></div></div></div><div id='method-getBuild' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-getBuild' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-getBuild' class='name expandable'>getBuild</a>( <span class='pre'></span> ) : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></div><div class='description'><div class='short'>Returns the build component value. ...</div><div class='long'><p>Returns the build component value.</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-getMajor' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-getMajor' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-getMajor' class='name expandable'>getMajor</a>( <span class='pre'></span> ) : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></div><div class='description'><div class='short'>Returns the major component value. ...</div><div class='long'><p>Returns the major component value.</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-getMinor' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-getMinor' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-getMinor' class='name expandable'>getMinor</a>( <span class='pre'></span> ) : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></div><div class='description'><div class='short'>Returns the minor component value. ...</div><div class='long'><p>Returns the minor component value.</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-getPatch' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-getPatch' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-getPatch' class='name expandable'>getPatch</a>( <span class='pre'></span> ) : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></div><div class='description'><div class='short'>Returns the patch component value. ...</div><div class='long'><p>Returns the patch component value.</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-getRelease' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-getRelease' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-getRelease' class='name expandable'>getRelease</a>( <span class='pre'></span> ) : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a></div><div class='description'><div class='short'>Returns the release component text (e.g., \"beta\"). ...</div><div class='long'><p>Returns the release component text (e.g., \"beta\").</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-getReleaseValue' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-getReleaseValue' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-getReleaseValue' class='name expandable'>getReleaseValue</a>( <span class='pre'></span> ) : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a>/<a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a></div><div class='description'><div class='short'>Returns the release component value for comparison purposes. ...</div><div class='long'><p>Returns the release component value for comparison purposes.</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a>/<a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-getShortVersion' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-getShortVersion' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-getShortVersion' class='name expandable'>getShortVersion</a>( <span class='pre'></span> ) : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a></div><div class='description'><div class='short'>Returns shortVersion version without dots and release ...</div><div class='long'><p>Returns shortVersion version without dots and release</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-gt' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-gt' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-gt' class='name expandable'>gt</a>( <span class='pre'>target</span> ) : <a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></div><div class='description'><div class='short'>Convenient alias to isGreaterThan ...</div><div class='long'><p>Convenient alias to <a href=\"#!/api/Ext.Version-method-isGreaterThan\" rel=\"Ext.Version-method-isGreaterThan\" class=\"docClass\">isGreaterThan</a></p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>target</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a>/<a href=\"#!/api/Ext.Version\" rel=\"Ext.Version\" class=\"docClass\">Ext.Version</a><div class='sub-desc'>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-gtEq' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-gtEq' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-gtEq' class='name expandable'>gtEq</a>( <span class='pre'>target</span> ) : <a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></div><div class='description'><div class='short'>Convenient alias to isGreaterThanOrEqual ...</div><div class='long'><p>Convenient alias to <a href=\"#!/api/Ext.Version-method-isGreaterThanOrEqual\" rel=\"Ext.Version-method-isGreaterThanOrEqual\" class=\"docClass\">isGreaterThanOrEqual</a></p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>target</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a>/<a href=\"#!/api/Ext.Version\" rel=\"Ext.Version\" class=\"docClass\">Ext.Version</a><div class='sub-desc'>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-isGreaterThan' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-isGreaterThan' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-isGreaterThan' class='name expandable'>isGreaterThan</a>( <span class='pre'>target</span> ) : <a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></div><div class='description'><div class='short'>Returns whether this version if greater than the supplied argument ...</div><div class='long'><p>Returns whether this version if greater than the supplied argument</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>target</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a><div class='sub-desc'><p>The version to compare with</p>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></span><div class='sub-desc'><p>True if this version if greater than the target, false otherwise</p>\n</div></li></ul></div></div></div><div id='method-isGreaterThanOrEqual' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-isGreaterThanOrEqual' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-isGreaterThanOrEqual' class='name expandable'>isGreaterThanOrEqual</a>( <span class='pre'>target</span> ) : <a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></div><div class='description'><div class='short'>Returns whether this version if greater than or equal to the supplied argument ...</div><div class='long'><p>Returns whether this version if greater than or equal to the supplied argument</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>target</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a><div class='sub-desc'><p>The version to compare with</p>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></span><div class='sub-desc'><p>True if this version if greater than or equal to the target, false otherwise</p>\n</div></li></ul></div></div></div><div id='method-isLessThan' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-isLessThan' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-isLessThan' class='name expandable'>isLessThan</a>( <span class='pre'>target</span> ) : <a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></div><div class='description'><div class='short'>Returns whether this version if smaller than the supplied argument ...</div><div class='long'><p>Returns whether this version if smaller than the supplied argument</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>target</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a><div class='sub-desc'><p>The version to compare with</p>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></span><div class='sub-desc'><p>True if this version if smaller than the target, false otherwise</p>\n</div></li></ul></div></div></div><div id='method-isLessThanOrEqual' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-isLessThanOrEqual' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-isLessThanOrEqual' class='name expandable'>isLessThanOrEqual</a>( <span class='pre'>target</span> ) : <a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></div><div class='description'><div class='short'>Returns whether this version if less than or equal to the supplied argument ...</div><div class='long'><p>Returns whether this version if less than or equal to the supplied argument</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>target</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a><div class='sub-desc'><p>The version to compare with</p>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></span><div class='sub-desc'><p>True if this version if less than or equal to the target, false otherwise</p>\n</div></li></ul></div></div></div><div id='method-lt' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-lt' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-lt' class='name expandable'>lt</a>( <span class='pre'>target</span> ) : <a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></div><div class='description'><div class='short'>Convenient alias to isLessThan ...</div><div class='long'><p>Convenient alias to <a href=\"#!/api/Ext.Version-method-isLessThan\" rel=\"Ext.Version-method-isLessThan\" class=\"docClass\">isLessThan</a></p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>target</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a>/<a href=\"#!/api/Ext.Version\" rel=\"Ext.Version\" class=\"docClass\">Ext.Version</a><div class='sub-desc'>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-ltEq' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-ltEq' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-ltEq' class='name expandable'>ltEq</a>( <span class='pre'>target</span> ) : <a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></div><div class='description'><div class='short'>Convenient alias to isLessThanOrEqual ...</div><div class='long'><p>Convenient alias to <a href=\"#!/api/Ext.Version-method-isLessThanOrEqual\" rel=\"Ext.Version-method-isLessThanOrEqual\" class=\"docClass\">isLessThanOrEqual</a></p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>target</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a>/<a href=\"#!/api/Ext.Version\" rel=\"Ext.Version\" class=\"docClass\">Ext.Version</a><div class='sub-desc'>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-match' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-match' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-match' class='name expandable'>match</a>( <span class='pre'>target</span> ) : <a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></div><div class='description'><div class='short'>Returns whether this version matches the supplied argument. ...</div><div class='long'><p>Returns whether this version matches the supplied argument. Example:</p>\n\n<pre><code>var version = new <a href=\"#!/api/Ext.Version\" rel=\"Ext.Version\" class=\"docClass\">Ext.Version</a>('1.0.2beta');\nconsole.log(version.match(1)); // True\nconsole.log(version.match(1.0)); // True\nconsole.log(version.match('1.0.2')); // True\nconsole.log(version.match('1.0.2RC')); // False\n</code></pre>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>target</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a>/<a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a><div class='sub-desc'><p>The version to compare with</p>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Boolean\" rel=\"Boolean\" class=\"docClass\">Boolean</a></span><div class='sub-desc'><p>True if this version matches the target, false otherwise</p>\n</div></li></ul></div></div></div><div id='method-toArray' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-toArray' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-toArray' class='name expandable'>toArray</a>( <span class='pre'></span> ) : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a>[]</div><div class='description'><div class='short'>Returns this format: [major, minor, patch, build, release]. ...</div><div class='long'><p>Returns this format: [major, minor, patch, build, release]. Useful for comparison.</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a>[]</span><div class='sub-desc'>\n</div></li></ul></div></div></div><div id='method-toString' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-toString' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-toString' class='name expandable'>toString</a>( <span class='pre'></span> ) : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a><strong class='private signature' >private</strong></div><div class='description'><div class='short'>Override the native toString method ...</div><div class='long'><p>Override the native toString method</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a></span><div class='sub-desc'><p>version</p>\n</div></li></ul></div></div></div><div id='method-valueOf' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-method-valueOf' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-method-valueOf' class='name expandable'>valueOf</a>( <span class='pre'></span> ) : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a><strong class='private signature' >private</strong></div><div class='description'><div class='short'>Override the native valueOf method ...</div><div class='long'><p>Override the native valueOf method</p>\n<h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a></span><div class='sub-desc'><p>version</p>\n</div></li></ul></div></div></div></div><div class='subsection'><div class='definedBy'>Defined By</div><h4 class='members-subtitle'>Static Methods</h3><div id='static-method-compare' class='member first-child not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-static-method-compare' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-static-method-compare' class='name expandable'>compare</a>( <span class='pre'>current, target</span> ) : <a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a><strong class='static signature' >static</strong></div><div class='description'><div class='short'>Compare 2 specified versions by ensuring the first parameter is a Version\ninstance and then calling the compareTo met...</div><div class='long'><p>Compare 2 specified versions by ensuring the first parameter is a <code>Version</code>\ninstance and then calling the <code>compareTo</code> method.</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>current</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a><div class='sub-desc'><p>The current version to compare to</p>\n</div></li><li><span class='pre'>target</span> : <a href=\"#!/api/String\" rel=\"String\" class=\"docClass\">String</a><div class='sub-desc'><p>The target version to compare to</p>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Number\" rel=\"Number\" class=\"docClass\">Number</a></span><div class='sub-desc'><p>Returns -1 if the current version is smaller than the target version, 1 if greater, and 0 if they're equivalent</p>\n</div></li></ul></div></div></div><div id='static-method-getComponentValue' class='member  not-inherited'><a href='#' class='side expandable'><span>&nbsp;</span></a><div class='title'><div class='meta'><span class='defined-in' rel='Ext.Version'>Ext.Version</span><br/><a href='source/Version.html#Ext-Version-static-method-getComponentValue' target='_blank' class='view-source'>view source</a></div><a href='#!/api/Ext.Version-static-method-getComponentValue' class='name expandable'>getComponentValue</a>( <span class='pre'>value</span> ) : <a href=\"#!/api/Object\" rel=\"Object\" class=\"docClass\">Object</a><strong class='static signature' >static</strong></div><div class='description'><div class='short'>Converts a version component to a comparable value ...</div><div class='long'><p>Converts a version component to a comparable value</p>\n<h3 class=\"pa\">Parameters</h3><ul><li><span class='pre'>value</span> : <a href=\"#!/api/Object\" rel=\"Object\" class=\"docClass\">Object</a><div class='sub-desc'><p>The value to convert</p>\n</div></li></ul><h3 class='pa'>Returns</h3><ul><li><span class='pre'><a href=\"#!/api/Object\" rel=\"Object\" class=\"docClass\">Object</a></span><div class='sub-desc'>\n</div></li></ul></div></div></div></div></div></div></div>","superclasses":[],"meta":{},"requires":[],"html_meta":{},"statics":{"property":[],"cfg":[],"css_var":[],"method":[{"tagname":"method","owner":"Ext.Version","meta":{"static":true},"name":"compare","id":"static-method-compare"},{"tagname":"method","owner":"Ext.Version","meta":{"static":true},"name":"getComponentValue","id":"static-method-getComponentValue"}],"event":[],"css_mixin":[]},"files":[{"href":"Version.html#Ext-Version","filename":"Version.js"}],"linenr":5,"members":{"property":[{"tagname":"property","owner":"Ext.Version","meta":{"readonly":true},"name":"build","id":"property-build"},{"tagname":"property","owner":"Ext.Version","meta":{"readonly":true},"name":"major","id":"property-major"},{"tagname":"property","owner":"Ext.Version","meta":{"readonly":true},"name":"minor","id":"property-minor"},{"tagname":"property","owner":"Ext.Version","meta":{"private":true,"readonly":true},"name":"parts","id":"property-parts"},{"tagname":"property","owner":"Ext.Version","meta":{"readonly":true},"name":"patch","id":"property-patch"},{"tagname":"property","owner":"Ext.Version","meta":{"readonly":true},"name":"release","id":"property-release"}],"cfg":[],"css_var":[],"method":[{"tagname":"method","owner":"Ext.Version","meta":{},"name":"constructor","id":"method-constructor"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"compareTo","id":"method-compareTo"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"equals","id":"method-equals"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"getBuild","id":"method-getBuild"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"getMajor","id":"method-getMajor"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"getMinor","id":"method-getMinor"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"getPatch","id":"method-getPatch"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"getRelease","id":"method-getRelease"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"getReleaseValue","id":"method-getReleaseValue"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"getShortVersion","id":"method-getShortVersion"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"gt","id":"method-gt"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"gtEq","id":"method-gtEq"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"isGreaterThan","id":"method-isGreaterThan"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"isGreaterThanOrEqual","id":"method-isGreaterThanOrEqual"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"isLessThan","id":"method-isLessThan"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"isLessThanOrEqual","id":"method-isLessThanOrEqual"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"lt","id":"method-lt"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"ltEq","id":"method-ltEq"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"match","id":"method-match"},{"tagname":"method","owner":"Ext.Version","meta":{},"name":"toArray","id":"method-toArray"},{"tagname":"method","owner":"Ext.Version","meta":{"private":true},"name":"toString","id":"method-toString"},{"tagname":"method","owner":"Ext.Version","meta":{"private":true},"name":"valueOf","id":"method-valueOf"}],"event":[],"css_mixin":[]},"inheritable":null,"private":null,"component":false,"name":"Ext.Version","singleton":false,"override":null,"inheritdoc":null,"id":"class-Ext.Version","mixins":[],"mixedInto":[]});